package Locale::Codes::Currency_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Wed May 30 10:22:19 EDT 2012

=pod

=head1 NAME

Locale::Codes::Currency_Retired - retired currency codes for the Locale::Codes::Currency module

=head1 SYNOPSIS

This module contains data used by the Locale::Codes::Currency module. It is
not intended to be used directly, and contains no calleable routines.

=head1 AUTHOR

See Locale::Codes for full author history.

Currently maintained by Sullivan Beck (sbeck@cpan.org).

=head1 COPYRIGHT

   Copyright (c) 1997-2001 Canon Research Centre Europe (CRE).
   Copyright (c) 2001-2010 Neil Bowers
   Copyright (c) 2010-2012 Sullivan Beck

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

use strict;
use warnings;
use utf8;
require 5.002;

our($VERSION);
$VERSION='3.22';

$Locale::Codes::Retired{'currency'}{'num'}{'code'} = {
};

$Locale::Codes::Retired{'currency'}{'alpha'}{'code'} = {
   q(YUM) => q(New Dinar),
   q(SKK) => q(Slovak Koruna),
   q(BYB) => q(Belarussian Ruble),
   q(MTL) => q(Maltese Lira),
   q(NLG) => q(Netherlands Guilder),
   q(ATS) => q(Schilling),
   q(XB5) => q(European Unit of Account 17),
   q(CYP) => q(Cyprus Pound),
   q(AFA) => q(Afghani),
   q(ZAL) => q(Financial Rand),
   q(FRF) => q(French Franc),
   q(ZRN) => q(New Zaire),
   q(TMM) => q(Manat),
   q(EEK) => q(Kroon),
   q(GHC) => q(Cedi),
   q(TJR) => q(Tajik Ruble),
   q(XFU) => q(UIC-Franc),
   q(AZM) => q(Azerbaijanian Manat),
   q(SRG) => q(Surinam Guilder),
   q(ESP) => q(Spanish Peseta),
   q(TPE) => q(Timor Escudo),
   q(GWP) => q(Guinea-Bissau Peso),
   q(LUF) => q(Luxembourg Franc),
   q(BGL) => q(Lev),
   q(ECV) => q(Unidad de Valor Constante (UVC)),
   q(PTE) => q(Portuguese Escudo),
   q(IEP) => q(Irish Pound),
   q(UAK) => q(Karbovanets),
   q(ECS) => q(Sucre),
   q(MZM) => q(Metical),
   q(RUR) => q(Russian Ruble),
   q(XFO) => q(Gold-Franc),
   q(BEF) => q(Belgian Franc),
   q(ROL) => q(Leu),
   q(MGF) => q(Malagasy Franc),
   q(AON) => q(New Kwanza),
   q(ZWD) => q(Zimbabwe Dollar),
   q(FIM) => q(Markka),
   q(SDD) => q(Sudanese Dinar),
   q(GRD) => q(Drachma),
   q(SIT) => q(Tolar),
   q(VEB) => q(Bolivar),
   q(AOR) => q(Kwanza Reajustado),
   q(ITL) => q(Italian Lira),
   q(TRL) => q(Turkish Lira),
   q(DEM) => q(German Mark),
   q(XEU) => q(ECU (until 1998-12-31)),
   q(ADP) => q(Andorran Peseta),
};

$Locale::Codes::Retired{'currency'}{'num'}{'name'} = {
   q(new manat) => [ q(934), q(New Manat) ],
   q(metical) => [ q(943), q(Metical) ],
   q(cedi) => [ q(936), q(Cedi) ],
   q(leu) => [ q(946), q(Leu) ],
};

$Locale::Codes::Retired{'currency'}{'alpha'}{'name'} = {
   q(unidades de formento) => [ q(CLF), q(Unidades de Formento) ],
   q(new leu) => [ q(RON), q(New Leu) ],
   q(schilling) => [ q(ATS), q(Schilling) ],
   q(netherlands antillian guilder) => [ q(ANG), q(Netherlands Antillian Guilder) ],
   q(irish pound) => [ q(IEP), q(Irish Pound) ],
   q(uic-franc) => [ q(XFU), q(UIC-Franc) ],
   q(new kwanza) => [ q(AON), q(New Kwanza) ],
   q(mexican nuevo peso) => [ q(MXN), q(Mexican Nuevo Peso) ],
   q(netherlands guilder) => [ q(NLG), q(Netherlands Guilder) ],
   q(cedi) => [ q(GHS), q(Cedi) ],
   q(unidad de valor constante (uvc)) => [ q(ECV), q(Unidad de Valor Constante (UVC)) ],
   q(karbovanets) => [ q(UAK), q(Karbovanets) ],
   q(european composite unit) => [ q(XBA), q(European Composite Unit) ],
   q(st. helena pound) => [ q(SHP), q(St. Helena Pound) ],
   q(ecu (until 1998-12-31)) => [ q(XEU), q(ECU (until 1998-12-31)) ],
   q(new manat) => [ q(TMT), q(New Manat) ],
   q(malagasy franc) => [ q(MGF), q(Malagasy Franc) ],
   q(kuna) => [ q(HRK), q(Kuna) ],
   q(european unit of account 17(e.u.a.-17)) => [ q(XBD), q(European Unit of Account 17(E.U.A.-17)) ],
   q(uruguay peso en unidades indexadas) => [ q(UYI), q(Uruguay Peso en Unidades Indexadas) ],
   q(german mark) => [ q(DEM), q(German Mark) ],
   q(timor escudo) => [ q(TPE), q(Timor Escudo) ],
   q(convertible marks) => [ q(BAM), q(Convertible Marks) ],
   q(lev) => [ q(BGL), q(Lev) ],
   q(candian dollar) => [ q(CAD), q(Candian Dollar) ],
   q(drachma) => [ q(GRD), q(Drachma) ],
   q(italian lira) => [ q(ITL), q(Italian Lira) ],
   q(european unit of account 17) => [ q(XB5), q(European Unit of Account 17) ],
   q(luxembourg franc) => [ q(LUF), q(Luxembourg Franc) ],
   q(french franc) => [ q(FRF), q(French Franc) ],
   q(sdr) => [ q(XDR), q(SDR) ],
   q(bolivar) => [ q(VEB), q(Bolivar) ],
   q(dinar) => [ q(BHD), q(Dinar) ],
   q(maltese lira) => [ q(MTL), q(Maltese Lira) ],
   q(tunisian dollar) => [ q(TND), q(Tunisian Dollar) ],
   q(markka) => [ q(FIM), q(Markka) ],
   q(andorran peseta) => [ q(ADP), q(Andorran Peseta) ],
   q(tajik ruble) => [ q(TJR), q(Tajik Ruble) ],
   q(tolar) => [ q(SIT), q(Tolar) ],
   q(metical) => [ q(MZN), q(Metical) ],
   q(aruban guilder) => [ q(AWG), q(Aruban Guilder) ],
   q(gold-franc) => [ q(XFO), q(Gold-Franc) ],
   q(cyprus pound) => [ q(CYP), q(Cyprus Pound) ],
   q(european monetary unit (e.m.u.-6)) => [ q(XBB), q(European Monetary Unit (E.M.U.-6)) ],
   q(kwanza reajustado) => [ q(AOR), q(Kwanza Reajustado) ],
   q(new zaire) => [ q(ZRN), q(New Zaire) ],
   q(kroon) => [ q(EEK), q(Kroon) ],
   q(belgian franc) => [ q(BEF), q(Belgian Franc) ],
   q(sudanese dinar) => [ q(SDD), q(Sudanese Dinar) ],
   q(spanish peseta) => [ q(ESP), q(Spanish Peseta) ],
   q(financial rand) => [ q(ZAL), q(Financial Rand) ],
   q(manat) => [ q(TMT), q(Manat) ],
   q(guinea-bissau peso) => [ q(GWP), q(Guinea-Bissau Peso) ],
   q(european unit of account 9) => [ q(XBC), q(European Unit of Account 9) ],
   q(european unit of account 9(e.u.a.-9)) => [ q(XBC), q(European Unit of Account 9(E.U.A.-9)) ],
   q(slovak koruna) => [ q(SKK), q(Slovak Koruna) ],
   q(shekel) => [ q(ILS), q(Shekel) ],
   q(portuguese escudo) => [ q(PTE), q(Portuguese Escudo) ],
   q(bond markets units european composite unit (eurco)) => [ q(XBA), q(Bond Markets Units European Composite Unit (EURCO)) ],
   q(surinam guilder) => [ q(SRG), q(Surinam Guilder) ],
   q(bermudian dollar (customarily known as bermuda dollar)) => [ q(BMD), q(Bermudian Dollar (customarily known as Bermuda Dollar)) ],
   q(new dinar) => [ q(YUM), q(New Dinar) ],
   q(franc congolais) => [ q(CDF), q(Franc Congolais) ],
   q(european monetary unit) => [ q(XBB), q(European Monetary Unit) ],
   q(leu) => [ q(RON), q(Leu) ],
};


1;
