package Locale::Codes::LangExt_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Wed May 30 10:22:19 EDT 2012

=pod

=head1 NAME

Locale::Codes::LangExt_Retired - retired langext codes for the Locale::Codes::LangExt module

=head1 SYNOPSIS

This module contains data used by the Locale::Codes::LangExt module. It is
not intended to be used directly, and contains no calleable routines.

=head1 AUTHOR

See Locale::Codes for full author history.

Currently maintained by Sullivan Beck (sbeck@cpan.org).

=head1 COPYRIGHT

   Copyright (c) 1997-2001 Canon Research Centre Europe (CRE).
   Copyright (c) 2001-2010 Neil Bowers
   Copyright (c) 2010-2012 Sullivan Beck

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

use strict;
use warnings;
use utf8;
require 5.002;

our($VERSION);
$VERSION='3.22';

$Locale::Codes::Retired{'langext'}{'alpha'}{'code'} = {
};

$Locale::Codes::Retired{'langext'}{'alpha'}{'name'} = {
};


1;
