package Locale::Codes::Language_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Wed May 30 10:22:19 EDT 2012

=pod

=head1 NAME

Locale::Codes::Language_Retired - retired language codes for the Locale::Codes::Language module

=head1 SYNOPSIS

This module contains data used by the Locale::Codes::Language module. It is
not intended to be used directly, and contains no calleable routines.

=head1 AUTHOR

See Locale::Codes for full author history.

Currently maintained by Sullivan Beck (sbeck@cpan.org).

=head1 COPYRIGHT

   Copyright (c) 1997-2001 Canon Research Centre Europe (CRE).
   Copyright (c) 2001-2010 Neil Bowers
   Copyright (c) 2010-2012 Sullivan Beck

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

use strict;
use warnings;
use utf8;
require 5.002;

our($VERSION);
$VERSION='3.22';

$Locale::Codes::Retired{'language'}{'alpha-2'}{'code'} = {
   q(jw) => q(Javanese),
   q(in) => q(Indonesian),
   q(mo) => q(Moldavian),
   q(iw) => q(Hebrew),
   q(ji) => q(Yiddish),
};

$Locale::Codes::Retired{'language'}{'alpha-3'}{'code'} = {
   q(bjq) => q(Southern Betsimisaraka Malagasy),
   q(dkl) => q(Kolum So Dogon),
   q(dha) => q(Dhanwar (India)),
   q(nbf) => q(Naxi),
   q(mja) => q(Mahei),
   q(tie) => q(Tingal),
   q(tkk) => q(Takpa),
   q(ayx) => q(Ayi (China)),
   q(noo) => q(Nootka),
};

$Locale::Codes::Retired{'language'}{'term'}{'code'} = {
};

$Locale::Codes::Retired{'language'}{'alpha-2'}{'name'} = {
   q(kirundi) => [ q(rn), q(Kirundi) ],
   q(latvian, lettish) => [ q(lv), q(Latvian, Lettish) ],
   q(greek, modern (1453-)) => [ q(el), q(Greek, Modern (1453-)) ],
   q(moldavian) => [ q(mo), q(Moldavian) ],
   q(frisian) => [ q(fy), q(Frisian) ],
   q(bihari) => [ q(bh), q(Bihari) ],
   q(malay) => [ q(ms), q(Malay) ],
   q(byelorussian) => [ q(be), q(Byelorussian) ],
   q(sotho, southern) => [ q(st), q(Sotho, Southern) ],
   q(norwegian bokmal) => [ q(nb), q(Norwegian Bokmal) ],
   q(greek) => [ q(el), q(Greek) ],
   q(faeroese) => [ q(fo), q(Faeroese) ],
   q((afan) oromo) => [ q(om), q((Afan) Oromo) ],
   q(laothian) => [ q(lo), q(Laothian) ],
   q(khmer) => [ q(km), q(Khmer) ],
   q(scots gaelic) => [ q(gd), q(Scots Gaelic) ],
   q(marshall) => [ q(mh), q(Marshall) ],
   q(bhutani) => [ q(dz), q(Bhutani) ],
   q(singhalese) => [ q(si), q(Singhalese) ],
   q(bengali; bangla) => [ q(bn), q(Bengali; Bangla) ],
   q(chichewa; nyanja) => [ q(ny), q(Chichewa; Nyanja) ],
   q(inupiak) => [ q(ik), q(Inupiak) ],
   q(greenlandic) => [ q(kl), q(Greenlandic) ],
   q(sangro) => [ q(sg), q(Sangro) ],
   q(ossetian; ossetic) => [ q(os), q(Ossetian; Ossetic) ],
   q(sami) => [ q(se), q(Sami) ],
   q(setswana) => [ q(tn), q(Setswana) ],
   q(punjabi) => [ q(pa), q(Punjabi) ],
   q(tegulu) => [ q(te), q(Tegulu) ],
   q(sotho) => [ q(st), q(Sotho) ],
   q(cambodian) => [ q(km), q(Cambodian) ],
   q(tonga) => [ q(to), q(Tonga) ],
   q(interlingua) => [ q(ia), q(Interlingua) ],
   q(rhaeto-romance) => [ q(rm), q(Rhaeto-Romance) ],
   q(sinhalese) => [ q(si), q(Sinhalese) ],
   q(gallegan) => [ q(gl), q(Gallegan) ],
   q(occitan) => [ q(oc), q(Occitan) ],
   q(swahili) => [ q(sw), q(Swahili) ],
   q(gaelic (scots)) => [ q(gd), q(Gaelic (Scots)) ],
   q(volapuk) => [ q(vo), q(Volapuk) ],
   q(siswati) => [ q(ss), q(Siswati) ],
   q(letzeburgesch) => [ q(lb), q(Letzeburgesch) ],
   q(pashto, pushto) => [ q(ps), q(Pashto, Pushto) ],
   q(sesotho) => [ q(st), q(Sesotho) ],
   q(fiji) => [ q(fj), q(Fiji) ],
};

$Locale::Codes::Retired{'language'}{'alpha-3'}{'name'} = {
   q(dhanwar (india)) => [ q(dha), q(Dhanwar (India)) ],
   q(ayi (china)) => [ q(ayx), q(Ayi (China)) ],
   q(hainyaxo bozo) => [ q(bzx), q(Hainyaxo Bozo) ],
   q(dutch, middle (ca.1050-1350)) => [ q(dum), q(Dutch, Middle (ca.1050-1350)) ],
   q(dogri) => [ q(doi), q(Dogri) ],
   q(kumak) => [ q(nee), q(Kumak) ],
   q(waray) => [ q(war), q(Waray) ],
   q(creoles and pidgins ) => [ q(crp), q(Creoles and pidgins ) ],
   q(kwato) => [ q(kop), q(Kwato) ],
   q(mende) => [ q(men), q(Mende) ],
   q(nung (myanmar)) => [ q(nun), q(Nung (Myanmar)) ],
   q(english, middle (1100-1500)) => [ q(enm), q(English, Middle (1100-1500)) ],
   q(dumpu) => [ q(wtf), q(Dumpu) ],
   q(ounge) => [ q(oue), q(Ounge) ],
   q(turkish, ottoman (1500-1928)) => [ q(ota), q(Turkish, Ottoman (1500-1928)) ],
   q(kag-fer-jiir-koor-ror-us-zuksun) => [ q(gel), q(Kag-Fer-Jiir-Koor-Ror-Us-Zuksun) ],
   q(creoles and pidgins, portuguese-based ) => [ q(cpp), q(Creoles and pidgins, Portuguese-based ) ],
   q(provencal, old (to 1500)) => [ q(pro), q(Provencal, Old (to 1500)) ],
   q(fang) => [ q(fan), q(Fang) ],
   q(miri) => [ q(mrg), q(Miri) ],
   q(yonggom) => [ q(yon), q(Yonggom) ],
   q(bemba) => [ q(bem), q(Bemba) ],
   q(kamba) => [ q(kam), q(Kamba) ],
   q(south american indian (other)) => [ q(sai), q(South American Indian (Other)) ],
   q(konkani) => [ q(kok), q(Konkani) ],
   q(creoles and pidgins, english based) => [ q(cpe), q(Creoles and pidgins, English based) ],
   q(nakama) => [ q(nib), q(Nakama) ],
   q(german, old high (ca.750-1050)) => [ q(goh), q(German, Old High (ca.750-1050)) ],
   q(basa) => [ q(bas), q(Basa) ],
   q(norse, old) => [ q(non), q(Norse, Old) ],
   q(gbaya) => [ q(gba), q(Gbaya) ],
   q(mahei) => [ q(mja), q(Mahei) ],
   q(german, middle high (ca.1050-1500)) => [ q(gmh), q(German, Middle High (ca.1050-1500)) ],
   q(duduela) => [ q(duk), q(Duduela) ],
   q(french, middle (ca.1400-1600)) => [ q(frm), q(French, Middle (ca.1400-1600)) ],
   q(burum-mindik) => [ q(bmu), q(Burum-Mindik) ],
   q(mbedam) => [ q(xmd), q(Mbedam) ],
   q(tingal) => [ q(tie), q(Tingal) ],
   q(kolum so dogon) => [ q(dkl), q(Kolum So Dogon) ],
   q(french, old (842-ca.1400)) => [ q(fro), q(French, Old (842-ca.1400)) ],
   q(ainu) => [ q(ain), q(Ainu) ],
   q(nootka) => [ q(noo), q(Nootka) ],
   q(irish, middle (900-1200)) => [ q(mga), q(Irish, Middle (900-1200)) ],
   q(irish, old (to 900)) => [ q(sga), q(Irish, Old (to 900)) ],
   q(mari) => [ q(chm), q(Mari) ],
   q(bekwil) => [ q(bkw), q(Bekwil) ],
   q(greek, ancient (to 1453)) => [ q(grc), q(Greek, Ancient (to 1453)) ],
   q(walamo) => [ q(wal), q(Walamo) ],
   q(bantu (other)) => [ q(bnt), q(Bantu (Other)) ],
   q(apalik) => [ q(apo), q(Apalik) ],
   q(creoles and pidgins, french-based ) => [ q(cpf), q(Creoles and pidgins, French-based ) ],
   q(persian, old (ca.600-400 b.c.)) => [ q(peo), q(Persian, Old (ca.600-400 B.C.)) ],
   q(takpa) => [ q(tkk), q(Takpa) ],
   q(english, old (ca.450-1100)) => [ q(ang), q(English, Old (ca.450-1100)) ],
   q(were) => [ q(wei), q(Were) ],
};

$Locale::Codes::Retired{'language'}{'term'}{'name'} = {
};


1;
