package Locale::Codes::Country_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Wed May 30 10:22:19 EDT 2012

=pod

=head1 NAME

Locale::Codes::Country_Retired - retired country codes for the Locale::Codes::Country module

=head1 SYNOPSIS

This module contains data used by the Locale::Codes::Country module. It is
not intended to be used directly, and contains no calleable routines.

=head1 AUTHOR

See Locale::Codes for full author history.

Currently maintained by Sullivan Beck (sbeck@cpan.org).

=head1 COPYRIGHT

   Copyright (c) 1997-2001 Canon Research Centre Europe (CRE).
   Copyright (c) 2001-2010 Neil Bowers
   Copyright (c) 2010-2012 Sullivan Beck

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

use strict;
use warnings;
use utf8;
require 5.002;

our($VERSION);
$VERSION='3.22';

$Locale::Codes::Retired{'country'}{'dom'}{'code'} = {
   q(FX) => q(France, Metropolitan),
   q(YU) => q(Yugoslavia ),
};

$Locale::Codes::Retired{'country'}{'alpha-2'}{'code'} = {
   q(zr) => q(Zaire),
   q(yu) => q(Yugoslavia),
   q(fx) => q(France, Metropolitan),
   q(cs) => q(Serbia and Montenegro),
   q(tp) => q(East Timor),
   q(an) => q(Netherlands Antilles),
};

$Locale::Codes::Retired{'country'}{'numeric'}{'code'} = {
   q(334) => q(Heard Island and Mcdonald Islands),
   q(074) => q(Bouvet Island),
   q(010) => q(Antarctica),
   q(086) => q(British Indian Ocean Territory),
   q(239) => q(South Georgia and the Islands),
   q(249) => q(France, Metropolitan),
   q(581) => q(United States Minor Outlying Islands),
   q(158) => q(Taiwan),
   q(166) => q(Cocos (Keeling) Islands),
   q(530) => q(Netherlands Antilles),
   q(162) => q(Christmas Island),
   q(891) => q(Serbia and Montenegro),
   q(736) => q(Sudan),
   q(260) => q(French Southern and Antarctic Lands),
};

$Locale::Codes::Retired{'country'}{'alpha-3'}{'code'} = {
   q(tmp) => q(East Timor),
   q(ant) => q(Netherlands Antilles),
   q(sgs) => q(South Georgia and the Islands),
   q(umi) => q(United States Minor Outlying Islands),
   q(cxr) => q(Christmas Island),
   q(rom) => q(Romania),
   q(iot) => q(British Indian Ocean Territory),
   q(bvt) => q(Bouvet Island),
   q(twn) => q(Taiwan),
   q(fxx) => q(France, Metropolitan),
   q(scg) => q(Serbia and Montenegro),
   q(hmd) => q(Heard Island and Mcdonald Islands),
   q(atf) => q(French Southern and Antarctic Lands),
   q(yug) => q(Yugoslavia),
   q(ata) => q(Antarctica),
   q(cck) => q(Cocos (Keeling) Islands),
   q(zar) => q(Zaire),
};

$Locale::Codes::Retired{'country'}{'fips-10'}{'code'} = {
   q(RN) => q(Saint Martin),
   q(UC) => q(Curacao),
   q(KV) => q(Kosovo),
   q(MJ) => q(Montenegro),
   q(RI) => q(Serbia),
   q(TB) => q(Saint Barthelemy),
   q(NN) => q(Sint Maarten),
};

$Locale::Codes::Retired{'country'}{'dom'}{'name'} = {
   q(libyan arab jamahiriya) => [ q(LY), q(Libyan Arab Jamahiriya) ],
   q(taiwan) => [ q(TW), q(Taiwan) ],
   q(soviet union ) => [ q(SU), q(Soviet Union ) ],
   q(yugoslavia ) => [ q(YU), q(Yugoslavia ) ],
   q(france, metropolitan) => [ q(FX), q(France, Metropolitan) ],
   q(bonaire, saint eustatius and saba) => [ q(BQ), q(Bonaire, Saint Eustatius and Saba) ],
   q(saint helena, ascension and tristan da cunha) => [ q(SH), q(Saint Helena, Ascension and Tristan da Cunha) ],
   q(saint martin) => [ q(MF), q(Saint Martin) ],
   q(portuguese timor ) => [ q(TP), q(Portuguese Timor ) ],
};

$Locale::Codes::Retired{'country'}{'alpha-2'}{'name'} = {
   q(netherlands antilles) => [ q(an), q(Netherlands Antilles) ],
   q(libyan arab jamahiriya) => [ q(ly), q(Libyan Arab Jamahiriya) ],
   q(virgin islands (u.s.)) => [ q(vi), q(Virgin Islands (U.S.)) ],
   q(east timor) => [ q(tl), q(East Timor) ],
   q(venezuela) => [ q(ve), q(Venezuela) ],
   q(kazakstan) => [ q(kz), q(Kazakstan) ],
   q(micronesia (federated states of)) => [ q(fm), q(Micronesia (Federated States of)) ],
   q(svalbard and jan mayen islands) => [ q(sj), q(Svalbard and Jan Mayen Islands) ],
   q(bosnia and herzegowina) => [ q(ba), q(Bosnia and Herzegowina) ],
   q(france, metropolitan) => [ q(fx), q(France, Metropolitan) ],
   q(yugoslavia) => [ q(yu), q(Yugoslavia) ],
   q(wallis and futuna islands) => [ q(wf), q(Wallis and Futuna Islands) ],
   q(heard and mc donald islands) => [ q(hm), q(Heard and Mc Donald Islands) ],
   q(st. pierre and miquelon) => [ q(pm), q(St. Pierre and Miquelon) ],
   q(zaire) => [ q(zr), q(Zaire) ],
   q(virgin islands (british)) => [ q(vg), q(Virgin Islands (British)) ],
   q(serbia and montenegro) => [ q(cs), q(Serbia and Montenegro) ],
   q(vatican city state (holy see)) => [ q(va), q(Vatican City State (Holy See)) ],
   q(st. helena) => [ q(sh), q(St. Helena) ],
   q(macau) => [ q(mo), q(Macau) ],
   q(bolivia) => [ q(bo), q(Bolivia) ],
   q(vietnam) => [ q(vn), q(Vietnam) ],
   q(saint helena) => [ q(sh), q(Saint Helena) ],
   q(saint martin) => [ q(mf), q(Saint Martin) ],
   q(iran (islamic republic of)) => [ q(ir), q(Iran (Islamic Republic of)) ],
};

$Locale::Codes::Retired{'country'}{'numeric'}{'name'} = {
   q(netherlands antilles) => [ q(530), q(Netherlands Antilles) ],
   q(libyan arab jamahiriya) => [ q(434), q(Libyan Arab Jamahiriya) ],
   q(east timor) => [ q(626), q(East Timor) ],
   q(united kingdom) => [ q(826), q(United Kingdom) ],
   q(venezuela) => [ q(862), q(Venezuela) ],
   q(taiwan) => [ q(158), q(Taiwan) ],
   q(antarctica) => [ q(010), q(Antarctica) ],
   q(taiwan, province of china) => [ q(158), q(Taiwan, Province of China) ],
   q(south georgia and the south sandwich islands) => [ q(239), q(South Georgia and the South Sandwich Islands) ],
   q(christmas island) => [ q(162), q(Christmas Island) ],
   q(korea, democratic people's republic of) => [ q(408), q(Korea, Democratic People's Republic of) ],
   q(heard island and mcdonald islands) => [ q(334), q(Heard Island and Mcdonald Islands) ],
   q(yugoslavia) => [ q(891), q(Yugoslavia) ],
   q(hong kong special administrative region of china) => [ q(344), q(Hong Kong Special Administrative Region of China) ],
   q(cocos (keeling) islands) => [ q(166), q(Cocos (Keeling) Islands) ],
   q(french southern territories) => [ q(260), q(French Southern Territories) ],
   q(virgin islands, british) => [ q(092), q(Virgin Islands, British) ],
   q(british indian ocean territory) => [ q(086), q(British Indian Ocean Territory) ],
   q(faroe islands) => [ q(234), q(Faroe Islands) ],
   q(zaire) => [ q(180), q(Zaire) ],
   q(congo, the democratic republic of the) => [ q(180), q(Congo, The Democratic Republic of the) ],
   q(united states minor outlying islands) => [ q(581), q(United States Minor Outlying Islands) ],
   q(iran, islamic republic of) => [ q(364), q(Iran, Islamic Republic of) ],
   q(serbia and montenegro) => [ q(891), q(Serbia and Montenegro) ],
   q(macedonia, the former yugoslav republic of) => [ q(807), q(Macedonia, the Former Yugoslav Republic of) ],
   q(macau) => [ q(446), q(Macau) ],
   q(french southern and antarctic lands) => [ q(260), q(French Southern and Antarctic Lands) ],
   q(hong kong) => [ q(344), q(Hong Kong) ],
   q(macao special administrative region of china) => [ q(446), q(Macao Special Administrative Region of China) ],
   q(palestinian territory, occupied) => [ q(275), q(Palestinian Territory, Occupied) ],
   q(kazakstan) => [ q(398), q(Kazakstan) ],
   q(macao) => [ q(446), q(Macao) ],
   q(france, metropolitan) => [ q(249), q(France, Metropolitan) ],
   q(tanzania, united republic of) => [ q(834), q(Tanzania, United Republic of) ],
   q(bouvet island) => [ q(074), q(Bouvet Island) ],
   q(holy see (vatican city state)) => [ q(336), q(Holy See (Vatican City State)) ],
   q(moldova, republic of) => [ q(498), q(Moldova, Republic of) ],
   q(micronesia, federated states of) => [ q(583), q(Micronesia, Federated States of) ],
   q(united states) => [ q(840), q(United States) ],
   q(virgin islands, u.s.) => [ q(850), q(Virgin Islands, U.S.) ],
   q(korea, republic of) => [ q(410), q(Korea, Republic of) ],
   q(bolivia) => [ q(068), q(Bolivia) ],
   q(vietnam) => [ q(704), q(Vietnam) ],
   q(south georgia and the islands) => [ q(239), q(South Georgia and the Islands) ],
   q(svalbard and jan mayen) => [ q(744), q(Svalbard and Jan Mayen) ],
   q(wallis and futuna) => [ q(876), q(Wallis and Futuna) ],
};

$Locale::Codes::Retired{'country'}{'alpha-3'}{'name'} = {
   q(netherlands antilles) => [ q(ant), q(Netherlands Antilles) ],
   q(libyan arab jamahiriya) => [ q(lby), q(Libyan Arab Jamahiriya) ],
   q(east timor) => [ q(tls), q(East Timor) ],
   q(united kingdom) => [ q(gbr), q(United Kingdom) ],
   q(venezuela) => [ q(ven), q(Venezuela) ],
   q(taiwan) => [ q(twn), q(Taiwan) ],
   q(antarctica) => [ q(ata), q(Antarctica) ],
   q(taiwan, province of china) => [ q(twn), q(Taiwan, Province of China) ],
   q(south georgia and the south sandwich islands) => [ q(sgs), q(South Georgia and the South Sandwich Islands) ],
   q(christmas island) => [ q(cxr), q(Christmas Island) ],
   q(korea, democratic people's republic of) => [ q(prk), q(Korea, Democratic People's Republic of) ],
   q(heard island and mcdonald islands) => [ q(hmd), q(Heard Island and Mcdonald Islands) ],
   q(yugoslavia) => [ q(yug), q(Yugoslavia) ],
   q(hong kong special administrative region of china) => [ q(hkg), q(Hong Kong Special Administrative Region of China) ],
   q(cocos (keeling) islands) => [ q(cck), q(Cocos (Keeling) Islands) ],
   q(french southern territories) => [ q(atf), q(French Southern Territories) ],
   q(virgin islands, british) => [ q(vgb), q(Virgin Islands, British) ],
   q(british indian ocean territory) => [ q(iot), q(British Indian Ocean Territory) ],
   q(faroe islands) => [ q(fro), q(Faroe Islands) ],
   q(zaire) => [ q(zar), q(Zaire) ],
   q(congo, the democratic republic of the) => [ q(cod), q(Congo, The Democratic Republic of the) ],
   q(united states minor outlying islands) => [ q(umi), q(United States Minor Outlying Islands) ],
   q(iran, islamic republic of) => [ q(irn), q(Iran, Islamic Republic of) ],
   q(serbia and montenegro) => [ q(scg), q(Serbia and Montenegro) ],
   q(macedonia, the former yugoslav republic of) => [ q(mkd), q(Macedonia, the Former Yugoslav Republic of) ],
   q(macau) => [ q(mac), q(Macau) ],
   q(french southern and antarctic lands) => [ q(atf), q(French Southern and Antarctic Lands) ],
   q(hong kong) => [ q(hkg), q(Hong Kong) ],
   q(macao special administrative region of china) => [ q(mac), q(Macao Special Administrative Region of China) ],
   q(palestinian territory, occupied) => [ q(pse), q(Palestinian Territory, Occupied) ],
   q(kazakstan) => [ q(kaz), q(Kazakstan) ],
   q(macao) => [ q(mac), q(Macao) ],
   q(france, metropolitan) => [ q(fxx), q(France, Metropolitan) ],
   q(tanzania, united republic of) => [ q(tza), q(Tanzania, United Republic of) ],
   q(bouvet island) => [ q(bvt), q(Bouvet Island) ],
   q(holy see (vatican city state)) => [ q(vat), q(Holy See (Vatican City State)) ],
   q(moldova, republic of) => [ q(mda), q(Moldova, Republic of) ],
   q(micronesia, federated states of) => [ q(fsm), q(Micronesia, Federated States of) ],
   q(united states) => [ q(usa), q(United States) ],
   q(virgin islands, u.s.) => [ q(vir), q(Virgin Islands, U.S.) ],
   q(korea, republic of) => [ q(kor), q(Korea, Republic of) ],
   q(bolivia) => [ q(bol), q(Bolivia) ],
   q(vietnam) => [ q(vnm), q(Vietnam) ],
   q(south georgia and the islands) => [ q(sgs), q(South Georgia and the Islands) ],
   q(svalbard and jan mayen) => [ q(sjm), q(Svalbard and Jan Mayen) ],
   q(wallis and futuna) => [ q(wlf), q(Wallis and Futuna) ],
};

$Locale::Codes::Retired{'country'}{'fips-10'}{'name'} = {
   q(islamic state of afghanistan) => [ q(AF), q(Islamic State of Afghanistan) ],
   q(territorial collectivity of saint pierre and miquelon) => [ q(SB), q(Territorial Collectivity of Saint Pierre and Miquelon) ],
   q(department of guiana) => [ q(FG), q(Department of Guiana) ],
   q(union of burma) => [ q(BM), q(Union of Burma) ],
   q(republic of iceland) => [ q(IC), q(Republic of Iceland) ],
   q(republic of cote d'ivoire) => [ q(IV), q(Republic of Cote D'Ivoire) ],
   q(federal democratic republic of ethiopia) => [ q(ET), q(Federal Democratic Republic of Ethiopia) ],
   q(arab republic of egypt) => [ q(EG), q(Arab Republic of Egypt) ],
   q(republic of tajikistan) => [ q(TI), q(Republic of Tajikistan) ],
   q(kingdom of bhutan) => [ q(BT), q(Kingdom of Bhutan) ],
   q(republic of el salvador) => [ q(ES), q(Republic of El Salvador) ],
   q(republic of guatemala) => [ q(GT), q(Republic of Guatemala) ],
   q(republic of niger) => [ q(NG), q(Republic of Niger) ],
   q(republic of guinea-bissau) => [ q(PU), q(Republic of Guinea-Bissau) ],
   q(republic of indonesia) => [ q(ID), q(Republic of Indonesia) ],
   q(republic of trinidad and tobago) => [ q(TD), q(Republic of Trinidad and Tobago) ],
   q(republic of vanuatu) => [ q(NH), q(Republic of Vanuatu) ],
   q(federative republic of brazil) => [ q(BR), q(Federative Republic of Brazil) ],
   q(republic of croatia) => [ q(HR), q(Republic of Croatia) ],
   q(virgin islands of the united states) => [ q(VQ), q(Virgin Islands of the United States) ],
   q(republic of honduras) => [ q(HO), q(Republic of Honduras) ],
   q(republic of chile) => [ q(CI), q(Republic of Chile) ],
   q(republic of cameroon) => [ q(CM), q(Republic of Cameroon) ],
   q(territorial collectivity of mayotte) => [ q(MF), q(Territorial Collectivity of Mayotte) ],
   q(kingdom of spain) => [ q(SP), q(Kingdom of Spain) ],
   q(republic of paraguay) => [ q(PA), q(Republic of Paraguay) ],
   q(kosovo) => [ q(KV), q(Kosovo) ],
   q(federal republic of germany) => [ q(GM), q(Federal Republic of Germany) ],
   q(republic of cyprus) => [ q(CY), q(Republic of Cyprus) ],
   q(hong kong special administrative region) => [ q(HK), q(Hong Kong Special Administrative Region) ],
   q(kingdom of norway) => [ q(NO), q(Kingdom of Norway) ],
   q(republic of suriname) => [ q(NS), q(Republic of Suriname) ],
   q(commonwealth of the northern mariana islands) => [ q(CQ), q(Commonwealth of the Northern Mariana Islands) ],
   q(swiss confederation) => [ q(SZ), q(Swiss Confederation) ],
   q(republic of djibouti) => [ q(DJ), q(Republic of Djibouti) ],
   q(republic of haiti) => [ q(HA), q(Republic of Haiti) ],
   q(republic of the gambia) => [ q(GA), q(Republic of the Gambia) ],
   q(islamic republic of mauritania) => [ q(MR), q(Islamic Republic of Mauritania) ],
   q(state of israel) => [ q(IS), q(State of Israel) ],
   q(united mexican states) => [ q(MX), q(United Mexican States) ],
   q(territory of cocos (keeling) islands) => [ q(CK), q(Territory of Cocos (Keeling) Islands) ],
   q(kingdom of tonga) => [ q(TN), q(Kingdom of Tonga) ],
   q(republic of malawi) => [ q(MI), q(Republic of Malawi) ],
   q(togolese republic) => [ q(TO), q(Togolese Republic) ],
   q(bolivarian republic of venezuela) => [ q(VE), q(Bolivarian Republic of Venezuela) ],
   q(republic of namibia) => [ q(WA), q(Republic of Namibia) ],
   q(republic of hungary) => [ q(HU), q(Republic of Hungary) ],
   q(republic of albania) => [ q(AL), q(Republic of Albania) ],
   q(commonwealth of puerto rico) => [ q(RQ), q(Commonwealth of Puerto Rico) ],
   q(principality of monaco) => [ q(MN), q(Principality of Monaco) ],
   q(socialist republic of vietnam) => [ q(VM), q(Socialist Republic of Vietnam) ],
   q(federation of saint kitts and nevis) => [ q(SC), q(Federation of Saint Kitts and Nevis) ],
   q(republic of estonia) => [ q(EN), q(Republic of Estonia) ],
   q(department of reunion) => [ q(RE), q(Department of Reunion) ],
   q(islamic republic of pakistan) => [ q(PK), q(Islamic Republic of Pakistan) ],
   q(kingdom of thailand) => [ q(TH), q(Kingdom of Thailand) ],
   q(kingdom of denmark) => [ q(DA), q(Kingdom of Denmark) ],
   q(saint barthelemy) => [ q(TB), q(Saint Barthelemy) ],
   q(territory of ashmore and cartier islands) => [ q(AT), q(Territory of Ashmore and Cartier Islands) ],
   q(republic of yemen) => [ q(YM), q(Republic of Yemen) ],
   q(republic of finland) => [ q(FI), q(Republic of Finland) ],
   q(republic of lithuania) => [ q(LH), q(Republic of Lithuania) ],
   q(republic of cape verde) => [ q(CV), q(Republic of Cape Verde) ],
   q(portuguese republic) => [ q(PO), q(Portuguese Republic) ],
   q(principality of andorra) => [ q(AN), q(Principality of Andorra) ],
   q(republic of senegal) => [ q(SG), q(Republic of Senegal) ],
   q(republic of austria) => [ q(AU), q(Republic of Austria) ],
   q(territory of guam) => [ q(GQ), q(Territory of Guam) ],
   q(republic of mozambique) => [ q(MZ), q(Republic of Mozambique) ],
   q(great socialist people's libyan arab jamahiriya) => [ q(LY), q(Great Socialist People's Libyan Arab Jamahiriya) ],
   q(republic of botswana) => [ q(BC), q(Republic of Botswana) ],
   q(republic of bolivia) => [ q(BL), q(Republic of Bolivia) ],
   q(territory of heard island and mcdonald islands) => [ q(HM), q(Territory of Heard Island and McDonald Islands) ],
   q(republic of kazakhstan) => [ q(KZ), q(Republic of Kazakhstan) ],
   q(department of martinique) => [ q(MB), q(Department of Martinique) ],
   q(oriental republic of uruguay) => [ q(UY), q(Oriental Republic of Uruguay) ],
   q(republic of costa rica) => [ q(CS), q(Republic of Costa Rica) ],
   q(state of eritrea) => [ q(ER), q(State of Eritrea) ],
   q(slovak republic) => [ q(LO), q(Slovak Republic) ],
   q(republic of belarus) => [ q(BO), q(Republic of Belarus) ],
   q(republic of angola) => [ q(AO), q(Republic of Angola) ],
   q(republic of panama) => [ q(PM), q(Republic of Panama) ],
   q(kingdom of swaziland) => [ q(WZ), q(Kingdom of Swaziland) ],
   q(state of qatar) => [ q(QA), q(State of Qatar) ],
   q(territory of new caledonia and dependencies) => [ q(NC), q(Territory of New Caledonia and Dependencies) ],
   q(republic of palau) => [ q(PS), q(Republic of Palau) ],
   q(territory of the french southern and antarctic lands) => [ q(FS), q(Territory of the French Southern and Antarctic Lands) ],
   q(democratic republic of sao tome and principe) => [ q(TP), q(Democratic Republic of Sao Tome and Principe) ],
   q(islamic republic of iran) => [ q(IR), q(Islamic Republic of Iran) ],
   q(montenegro) => [ q(MJ), q(Montenegro) ],
   q(commonwealth of australia) => [ q(AS), q(Commonwealth of Australia) ],
   q(people's republic of china) => [ q(CH), q(People's Republic of China) ],
   q(republic of sierra leone) => [ q(SL), q(Republic of Sierra Leone) ],
   q(republic of the fiji islands) => [ q(FJ), q(Republic of the Fiji Islands) ],
   q(serbia) => [ q(RI), q(Serbia) ],
   q(territory of norfolk island) => [ q(NF), q(Territory of Norfolk Island) ],
   q(republic of zambia) => [ q(ZA), q(Republic of Zambia) ],
   q(republic of tunisia) => [ q(TS), q(Republic of Tunisia) ],
   q(republic of peru) => [ q(PE), q(Republic of Peru) ],
   q(republic of slovenia) => [ q(SI), q(Republic of Slovenia) ],
   q(republic of the congo) => [ q(CF), q(Republic of the Congo) ],
   q(republic of liberia) => [ q(LI), q(Republic of Liberia) ],
   q(republic of equatorial guinea) => [ q(EK), q(Republic of Equatorial Guinea) ],
   q(macau special administrative region) => [ q(MC), q(Macau Special Administrative Region) ],
   q(republic of san marino) => [ q(SM), q(Republic of San Marino) ],
   q(republic of guinea) => [ q(GV), q(Republic of Guinea) ],
   q(commonwealth of dominica) => [ q(DO), q(Commonwealth of Dominica) ],
   q(federal republic of nigeria) => [ q(NI), q(Federal Republic of Nigeria) ],
   q(italian republic) => [ q(IT), q(Italian Republic) ],
   q(kingdom of cambodia) => [ q(CB), q(Kingdom of Cambodia) ],
   q(pitcairn, henderson, ducie and oeno islands) => [ q(PC), q(Pitcairn, Henderson, Ducie and Oeno Islands) ],
   q(kingdom of belgium) => [ q(BE), q(Kingdom of Belgium) ],
   q(kingdom of sweden) => [ q(SW), q(Kingdom of Sweden) ],
   q(bailiwick of guernsey) => [ q(GK), q(Bailiwick of Guernsey) ],
   q(democratic socialist republic of sri lanka) => [ q(CE), q(Democratic Socialist Republic of Sri Lanka) ],
   q(republic of maldives) => [ q(MV), q(Republic of Maldives) ],
   q(republic of mali) => [ q(ML), q(Republic of Mali) ],
   q(republic of south africa) => [ q(SF), q(Republic of South Africa) ],
   q(sultanate of oman) => [ q(MU), q(Sultanate of Oman) ],
   q(territory of american samoa) => [ q(AQ), q(Territory of American Samoa) ],
   q(rwandese republic) => [ q(RW), q(Rwandese Republic) ],
   q(republic of latvia) => [ q(LG), q(Republic of Latvia) ],
   q(independent state of papua new guinea) => [ q(PP), q(Independent State of Papua New Guinea) ],
   q(republic of nicaragua) => [ q(NU), q(Republic of Nicaragua) ],
   q(republic of the philippines) => [ q(RP), q(Republic of the Philippines) ],
   q(french republic) => [ q(FR), q(French Republic) ],
   q(grand duchy of luxembourg) => [ q(LU), q(Grand Duchy of Luxembourg) ],
   q(bailiwick of jersey) => [ q(JE), q(Bailiwick of Jersey) ],
   q(republic of macedonia) => [ q(MK), q(Republic of Macedonia) ],
   q(state of the vatican city) => [ q(VT), q(State of the Vatican City) ],
   q(people's democratic republic of algeria) => [ q(AG), q(People's Democratic Republic of Algeria) ],
   q(republic of kiribati) => [ q(KR), q(Republic of Kiribati) ],
   q(republic of cuba) => [ q(CU), q(Republic of Cuba) ],
   q(kingdom of nepal) => [ q(NP), q(Kingdom of Nepal) ],
   q(saint martin) => [ q(RN), q(Saint Martin) ],
   q(republic of the marshall islands) => [ q(RM), q(Republic of the Marshall Islands) ],
   q(kingdom of the netherlands) => [ q(NL), q(Kingdom of the Netherlands) ],
   q(republic of madagascar) => [ q(MA), q(Republic of Madagascar) ],
   q(people's republic of bangladesh) => [ q(BG), q(People's Republic of Bangladesh) ],
   q(kingdom of morocco) => [ q(MO), q(Kingdom of Morocco) ],
   q(republic of the sudan) => [ q(SU), q(Republic of the Sudan) ],
   q(co-operative republic of guyana) => [ q(GY), q(Co-operative Republic of Guyana) ],
   q(republic of ghana) => [ q(GH), q(Republic of Ghana) ],
   q(state of bahrain) => [ q(BA), q(State of Bahrain) ],
   q(republic of chad) => [ q(CD), q(Republic of Chad) ],
   q(gabonese republic) => [ q(GB), q(Gabonese Republic) ],
   q(coral sea islands territory) => [ q(CR), q(Coral Sea Islands Territory) ],
   q(republic of zimbabwe) => [ q(ZI), q(Republic of Zimbabwe) ],
   q(republic of poland) => [ q(PL), q(Republic of Poland) ],
   q(argentine republic) => [ q(AR), q(Argentine Republic) ],
   q(republic of malta) => [ q(MT), q(Republic of Malta) ],
   q(republic of azerbaijan) => [ q(AJ), q(Republic of Azerbaijan) ],
   q(kyrgyz republic) => [ q(KG), q(Kyrgyz Republic) ],
   q(hellenic republic) => [ q(GR), q(Hellenic Republic) ],
   q(kingdom of saudi arabia) => [ q(SA), q(Kingdom of Saudi Arabia) ],
   q(republic of armenia) => [ q(AM), q(Republic of Armenia) ],
   q(republic of benin) => [ q(BN), q(Republic of Benin) ],
   q(curacao) => [ q(UC), q(Curacao) ],
   q(republic of ecuador) => [ q(EC), q(Republic of Ecuador) ],
   q(territory of french polynesia) => [ q(FP), q(Territory of French Polynesia) ],
   q(republic of lesotho) => [ q(LT), q(Republic of Lesotho) ],
   q(hashemite kingdom of jordan) => [ q(JO), q(Hashemite Kingdom of Jordan) ],
   q(republic of burundi) => [ q(BY), q(Republic of Burundi) ],
   q(republic of uzbekistan) => [ q(UZ), q(Republic of Uzbekistan) ],
   q(territory of the wallis and futuna islands) => [ q(WF), q(Territory of the Wallis and Futuna Islands) ],
   q(sint maarten) => [ q(NN), q(Sint Maarten) ],
   q(state of kuwait) => [ q(KU), q(State of Kuwait) ],
   q(lebanese republic) => [ q(LE), q(Lebanese Republic) ],
   q(republic of colombia) => [ q(CO), q(Republic of Colombia) ],
   q(department of guadeloupe) => [ q(GP), q(Department of Guadeloupe) ],
   q(republic of singapore) => [ q(SN), q(Republic of Singapore) ],
   q(republic of turkey) => [ q(TU), q(Republic of Turkey) ],
   q(republic of iraq) => [ q(IZ), q(Republic of Iraq) ],
   q(republic of seychelles) => [ q(SE), q(Republic of Seychelles) ],
   q(independent state of samoa) => [ q(WS), q(Independent State of Samoa) ],
   q(republic of india) => [ q(IN), q(Republic of India) ],
   q(territory of christmas island) => [ q(KT), q(Territory of Christmas Island) ],
   q(republic of nauru) => [ q(NR), q(Republic of Nauru) ],
   q(negara brunei darussalam) => [ q(BX), q(Negara Brunei Darussalam) ],
   q(federal islamic republic of the comoros) => [ q(CN), q(Federal Islamic Republic of the Comoros) ],
   q(republic of mauritius) => [ q(MP), q(Republic of Mauritius) ],
   q(commonwealth of the bahamas) => [ q(BF), q(Commonwealth of the Bahamas) ],
   q(principality of liechtenstein) => [ q(LS), q(Principality of Liechtenstein) ],
   q(republic of kenya) => [ q(KE), q(Republic of Kenya) ],
};


1;
