/* -*- buffer-read-only: t -*-
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 * This file is built by regen/mk_invlists.pl from Unicode::UCD.
 * Any changes made here will be lost!
 */

/* See the generating file for comments */


UV Latin1_invlist[] = {
	2,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	0,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	256,
	0
};

UV AboveLatin1_invlist[] = {
	1,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	256
};

UV ASCII_invlist[] = {
	2,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	0,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	128,
	0
};

UV L1Cased_invlist[] = {
	16,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	65,
	91,
	97,
	123,
	170,
	171,
	181,
	182,
	186,
	187,
	192,
	215,
	216,
	247,
	248,
	443
};

UV VertSpace_invlist[] = {
	6,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	10,
	14,
	133,
	134,
	8232,
	8234
};

UV PerlSpace_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	9,
	14,
	32,
	33
};

UV XPerlSpace_invlist[] = {
	22,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	9,
	14,
	32,
	33,
	133,
	134,
	160,
	161,
	5760,
	5761,
	6158,
	6159,
	8192,
	8203,
	8232,
	8234,
	8239,
	8240,
	8287,
	8288,
	12288,
	12289
};

UV PosixAlnum_invlist[] = {
	6,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	48,
	58,
	65,
	91,
	97,
	123
};

UV L1PosixAlnum_invlist[] = {
	18,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	48,
	58,
	65,
	91,
	97,
	123,
	170,
	171,
	181,
	182,
	186,
	187,
	192,
	215,
	216,
	247,
	248,
	706
};

UV PosixAlpha_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	65,
	91,
	97,
	123
};

UV L1PosixAlpha_invlist[] = {
	16,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	65,
	91,
	97,
	123,
	170,
	171,
	181,
	182,
	186,
	187,
	192,
	215,
	216,
	247,
	248,
	706
};

UV PosixBlank_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	9,
	10,
	32,
	33
};

UV XPosixBlank_invlist[] = {
	18,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	9,
	10,
	32,
	33,
	160,
	161,
	5760,
	5761,
	6158,
	6159,
	8192,
	8203,
	8239,
	8240,
	8287,
	8288,
	12288,
	12289
};

UV PosixCntrl_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	0,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	32,
	127,
	128,
	0
};

UV XPosixCntrl_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	0,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	32,
	127,
	160,
	0
};

UV PosixDigit_invlist[] = {
	2,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	48,
	58
};

UV PosixGraph_invlist[] = {
	2,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	33,
	127
};

UV L1PosixGraph_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	33,
	127,
	161,
	888
};

UV PosixLower_invlist[] = {
	2,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	97,
	123
};

UV L1PosixLower_invlist[] = {
	12,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	97,
	123,
	170,
	171,
	181,
	182,
	186,
	187,
	223,
	247,
	248,
	256
};

UV PosixPrint_invlist[] = {
	2,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	32,
	127
};

UV L1PosixPrint_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	32,
	127,
	160,
	888
};

UV PosixPunct_invlist[] = {
	8,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	33,
	48,
	58,
	65,
	91,
	97,
	123,
	127
};

UV L1PosixPunct_invlist[] = {
	20,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	33,
	48,
	58,
	65,
	91,
	97,
	123,
	127,
	161,
	162,
	167,
	168,
	171,
	172,
	182,
	184,
	187,
	188,
	191,
	192
};

UV PosixSpace_invlist[] = {
	4,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	9,
	14,
	32,
	33
};

UV XPosixSpace_invlist[] = {
	22,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	9,
	14,
	32,
	33,
	133,
	134,
	160,
	161,
	5760,
	5761,
	6158,
	6159,
	8192,
	8203,
	8232,
	8234,
	8239,
	8240,
	8287,
	8288,
	12288,
	12289
};

UV PosixUpper_invlist[] = {
	2,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	65,
	91
};

UV L1PosixUpper_invlist[] = {
	6,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	65,
	91,
	192,
	215,
	216,
	223
};

UV PosixWord_invlist[] = {
	8,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	48,
	58,
	65,
	91,
	95,
	96,
	97,
	123
};

UV L1PosixWord_invlist[] = {
	20,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	48,
	58,
	65,
	91,
	95,
	96,
	97,
	123,
	170,
	171,
	181,
	182,
	186,
	187,
	192,
	215,
	216,
	247,
	248,
	706
};

UV PosixXDigit_invlist[] = {
	6,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	48,
	58,
	65,
	71,
	97,
	103
};

UV XPosixXDigit_invlist[] = {
	12,	/* Number of elements */
	0,	/* Current iteration position */
	1064334010, /* Version and data structure type */
	1,	/* 0 if this is the first element of the list proper;
		   1 if the next element is the first */
	48,
	58,
	65,
	71,
	97,
	103,
	65296,
	65306,
	65313,
	65319,
	65345,
	65351
};

/* ex: set ro: */
