+{
   locale_version => 0.88,
# b-hook, d-hook, k-hook, y-hook don't require tailoring
   entry => <<'ENTRY', # for DUCET v6.1.0
0073 0068 ; [.17D9.0020.0002.0073] # <LATIN SMALL LETTER S, LATIN SMALL LETTER H>
0053 0068 ; [.17D9.0020.0007.0053] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER H>
0053 0048 ; [.17D9.0020.0008.0053] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER H>
0074 0073 ; [.17FC.0020.0002.0074] # <LATIN SMALL LETTER T, LATIN SMALL LETTER S>
0054 0073 ; [.17FC.0020.0007.0054] # <LATIN CAPITAL LETTER T, LATIN SMALL LETTER S>
0054 0053 ; [.17FC.0020.0008.0054] # <LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER S>
02BC 0079 ; [.1871.0020.0003.02BC] # <MODIFIER LETTER APOSTROPHE, LATIN SMALL LETTER Y>
02BC 0059 ; [.1871.0020.0009.02BC] # <MODIFIER LETTER APOSTROPHE, LATIN CAPITAL LETTER Y>
0027 0079 ; [.1871.0020.0004.0027] # <APOSTROPHE, LATIN SMALL LETTER Y>
0027 0059 ; [.1871.0020.000A.0027] # <APOSTROPHE, LATIN CAPITAL LETTER Y>
ENTRY
};
