+{
   locale_version => 0.88,
   upper_before_lower => 1,
   entry => <<'ENTRY', # for DUCET v6.1.0
010B      ; [.1601.0020.0002.010B] # LATIN SMALL LETTER C WITH DOT ABOVE
0063 0307 ; [.1601.0020.0002.010B] # LATIN SMALL LETTER C WITH DOT ABOVE
010A      ; [.1601.0020.0008.010A] # LATIN CAPITAL LETTER C WITH DOT ABOVE
0043 0307 ; [.1601.0020.0008.010A] # LATIN CAPITAL LETTER C WITH DOT ABOVE
0121      ; [.1675.0020.0002.0121] # LATIN SMALL LETTER G WITH DOT ABOVE
0067 0307 ; [.1675.0020.0002.0121] # LATIN SMALL LETTER G WITH DOT ABOVE
0120      ; [.1675.0020.0008.0120] # LATIN CAPITAL LETTER G WITH DOT ABOVE
0047 0307 ; [.1675.0020.0008.0120] # LATIN CAPITAL LETTER G WITH DOT ABOVE
0067 0127 ; [.1698.0020.0002.0067] # <LATIN SMALL LETTER G, LATIN SMALL LETTER H WITH STROKE>
0067 0126 ; [.1698.0020.0007.0067][.0000.0000.0002.0000] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER H WITH STROKE>
0047 0127 ; [.1698.0020.0007.0047][.0000.0000.0008.0000] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER H WITH STROKE>
0047 0126 ; [.1698.0020.0008.0047] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER H WITH STROKE>
0067 0068 0335 ; [.1698.0020.0002.0067] # <LATIN SMALL LETTER G, LATIN SMALL LETTER H, COMBINING SHORT STROKE OVERLAY>
0067 0048 0335 ; [.1698.0020.0007.0067][.0000.0000.0002.0000] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER H, COMBINING SHORT STROKE OVERLAY>
0047 0068 0335 ; [.1698.0020.0007.0047][.0000.0000.0008.0000] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER H, COMBINING SHORT STROKE OVERLAY>
0047 0048 0335 ; [.1698.0020.0008.0047] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER H, COMBINING SHORT STROKE OVERLAY>
0127      ; [.16B1.0020.0002.0127] # LATIN SMALL LETTER H WITH STROKE
0068 0335 ; [.16B1.0020.0002.0127] # LATIN SMALL LETTER H WITH STROKE
0126      ; [.16B1.0020.0008.0126] # LATIN CAPITAL LETTER H WITH STROKE
0048 0335 ; [.16B1.0020.0008.0126] # LATIN CAPITAL LETTER H WITH STROKE
017C      ; [.1879.0020.0002.017C] # LATIN SMALL LETTER Z WITH DOT ABOVE
007A 0307 ; [.1879.0020.0002.017C] # LATIN SMALL LETTER Z WITH DOT ABOVE
017B      ; [.1879.0020.0008.017B] # LATIN CAPITAL LETTER Z WITH DOT ABOVE
005A 0307 ; [.1879.0020.0008.017B] # LATIN CAPITAL LETTER Z WITH DOT ABOVE
ENTRY
};
